{-------------------------------------------------------------------------------
  Demo: printing documents using TRVReportHelper.
  This demo loads RVF file in rvh.RichView and prints it in two columns.
  This demo is similar to the demo in Demos\Delphi\Assorted\Printing\ReportHelper,
  but provides a print preview.

  Document is loaded in rvh: TRVReportHelper.
  RVPrint1: TRVPrint prints document from rvHidden: TRichView.
  rvHidden contains one empty page. All actual drawing is in
  RVPrint1.OnPagePrepaint.

-------------------------------------------------------------------------------}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Printers, Dialogs,
  RVScroll, RichView, PtblRV, RVReport, StdCtrls, RVStyle, CRVPP, RVPP;

type
  TForm1 = class(TForm)
    Button1: TButton;
    rvh: TRVReportHelper;
    RVStyle1: TRVStyle;
    OpenDialog1: TOpenDialog;
    Button3: TButton;
    RVPrint1: TRVPrint;
    RVPrintPreview1: TRVPrintPreview;
    rvHidden: TRichView;
    RVStyle3: TRVStyle;
    ScrollBar1: TScrollBar;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
      Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
    procedure ScrollBar1Change(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    PageNo: Integer; // page index for displaying in print preview, changed with scrollbar
    PrinterCanvas: TCanvas; // printer-compatible canvas for formatting rvh
    procedure PreparePrinting;
    function GetColumnWidth(PageWidth: Integer): Integer;
    procedure CreatePrinterCanvas;
    procedure DestroyPrinterCanvas;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  with rvh.RichView do begin
    RVFOptions := [rvfoSavePicturesBody,rvfoSaveControlsBody,rvfoSaveBinary,
      rvfoSaveBack,rvfoLoadBack,rvfoSaveTextStyles,rvfoSaveParaStyles,
      rvfoSaveLayout,rvfoLoadLayout];
    Options := Options + [rvoTagsArePChars];
    RVFParaStylesReadMode := rvf_sInsertMerge;
    RVFTextStylesReadMode := rvf_sInsertMerge;
    Style := RVStyle1;
  end;
  rvHidden.AddNL('', 0, 0); // rvHidden must not be empty, or RVPrint fails
  RVPrint1.AssignSource(rvHidden);
  RVPrint1.FormatPages(rvdoAll);
  RVPrintPreview1.UpdateView;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  DestroyPrinterCanvas;
end;

// Opening document and formatting it for printing
procedure TForm1.Button1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    rvh.RichView.Clear;
    rvh.RichView.LoadRVF(OpenDialog1.FileName);
    PreparePrinting;
  end;
end;

// Event on painting page
// We ignore PageNo parameter of this method (since rvHidden is printed, and it
// contains only one empty page, this parameter is always 1).
// In this method, we use Self.PageNo
procedure TForm1.RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
  Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
var ColWidth, ColHeight, Col1Left, Col2Left, TextWidth,
    ColNo: Integer;
    s: String;
begin
  // calculating columns width and positions
  ColWidth := GetColumnWidth(PrintAreaRect.Right-PrintAreaRect.Left);
  ColHeight := PrintAreaRect.Bottom-PrintAreaRect.Top;
  Col1Left := PrintAreaRect.Left;
  Col2Left := PrintAreaRect.Right-ColWidth;
  ColNo := Self.PageNo * 2 - 1;
  // drawing columns
  if ColNo<=rvh.PagesCount then
    rvh.DrawPageAt(Col1Left, PrintAreaRect.Top, ColNo, Canvas, Preview, ColHeight);
  if ColNo+1<=rvh.PagesCount then
    rvh.DrawPageAt(Col2Left, PrintAreaRect.Top, ColNo+1, Canvas, Preview, ColHeight);
  // drawing footer
  s := '- '+IntToStr(Self.PageNo)+' - ';
  Canvas.Font.Name := 'Arial';
  Canvas.Font.Size := 10;
  Canvas.Font.Color := clBlack;
  Canvas.Font.Style := [];
  Canvas.Brush.Style := bsClear;
  TextWidth := Canvas.TextWidth(s);
  Canvas.TextOut((PrintAreaRect.Right+PrintAreaRect.Left-TextWidth) div 2, PrintAreaRect.Bottom, s);
end;

// Returns width and height of printable area on page, minus RVPrint's margins
procedure GetPageSize(RVPrint: TRVPrint; DC: HDC; var Width, Height: Integer);
var
  phoX, phoY, phW, phH, lpy, lpx, LM, TM, RM, BM: Integer;
begin

  Width  := GetDeviceCaps(DC, HORZRES);
  Height := GetDeviceCaps(DC, VERTRES);

  lpy := GetDeviceCaps(DC, LOGPIXELSY);
  lpx := GetDeviceCaps(DC, LOGPIXELSX);

  phoX := GetDeviceCaps(DC, PHYSICALOFFSETX);
  phoY := GetDeviceCaps(DC, PHYSICALOFFSETY);
  phW  := GetDeviceCaps(DC, PHYSICALWIDTH);
  phH  := GetDeviceCaps(DC, PHYSICALHEIGHT);

  LM := MulDiv(RVPrint.LeftMarginMM,   5*lpx, 127)- phoX;
  TM := MulDiv(RVPrint.TopMarginMM,    5*lpy, 127)- phoY;
  RM := MulDiv(RVPrint.RightMarginMM,  5*lpx, 127)- (phW-(phoX+Width));
  BM := MulDiv(RVPrint.BottomMarginMM, 5*lpy, 127)- (phH-(phoY+Height));

  if LM<0 then LM := 0;
  if TM<0 then TM := 0;
  if RM<0 then RM := 0;
  if BM<0 then BM := 0;

  dec(Width, LM+RM);
  dec(Height, TM+BM);

end;

// Preparing document in rvh for printing in 2 columns.
// This method:
// - initiates PrinterCanvas (must be valid till print/preview is finished);
// - formats document in rvh
// - assigns scrollbar
procedure TForm1.PreparePrinting;
var w, h, ColWidth, PageCount: Integer;
begin
  CreatePrinterCanvas;
  GetPageSize(RVPrint1, PrinterCanvas.Handle, w, h);
  ColWidth := GetColumnWidth(w);
  rvh.Init(PrinterCanvas, ColWidth);
  while rvh.FormatNextPage(h) do;
  ScrollBar1.Position := 1;
  PageCount := (rvh.PagesCount+1) div 2;
  if PageCount=0 then
    PageCount := 1;
  ScrollBar1.Max := PageCount;
  ScrollBar1Change(nil);
end;

// Returns column width (less than 1/2 of PageWidth)
function TForm1.GetColumnWidth(PageWidth: Integer): Integer;
begin
  Result := PageWidth * 5 div 11;
end;
// When scrollbar position is changed, changing PageNo
procedure TForm1.ScrollBar1Change(Sender: TObject);
begin
  PageNo := ScrollBar1.Position;
  RVPrintPreview1.Refresh;
end;

// Creates canvas for formatting rvh
procedure TForm1.CreatePrinterCanvas;
begin
  DestroyPrinterCanvas;
  PrinterCanvas := TCanvas.Create;
  PrinterCanvas.Handle := RV_GetPrinterDC;
  PrinterCanvas.Font.PixelsPerInch := GetDeviceCaps(PrinterCanvas.Handle, LOGPIXELSY);
end;
// Destroys canvas created in CreatePrinterCanvas
procedure TForm1.DestroyPrinterCanvas;
var DC: HDC;
begin
  if PrinterCanvas=nil then
    exit;
  DC := PrinterCanvas.Handle;
  PrinterCanvas.Handle := 0;
  DeleteDC(DC);
  FreeAndNil(PrinterCanvas);
end;

// Printing
procedure TForm1.Button3Click(Sender: TObject);
var i, PrevewPageNo: Integer;
begin
  PrevewPageNo := PageNo;
  Printer.Title := '2 Columns Demo';
  Printer.BeginDoc;
  try
    for i := 1 to (rvh.PagesCount+1) div 2 do begin
      if i>1 then
        Printer.NewPage;
      PageNo := i;
      RVPrint1.ContinuousPrint; // since rvHidden contains one empty page,
                                // this method prints this page
    end;
  finally
    Printer.EndDoc;
    PageNo := PrevewPageNo;
  end;
end;

end.
